#ifndef NICO_DELTA_CHECKSUM
#define NICO_DELTA_CHECKSUM

action v4sum() {
    bit<32> tmp = 0;
    bit<32> carryover = 0;

    tmp = tmp + (bit<32>) hdr.ipv4.src_addr[15:0];              // 16 bit
    tmp = tmp + (bit<32>) hdr.ipv4.src_addr[31:16];             // 16 bit
    tmp = tmp + (bit<32>) hdr.ipv4.dst_addr[15:0];              // 16 bit
    tmp = tmp + (bit<32>) hdr.ipv4.dst_addr[31:16];             // 16 bit

    tmp = tmp + (bit<32>) hdr.ipv4.totalLen -20;                // 16 bit
    tmp = tmp + (bit<32>) hdr.ipv4.protocol;                    // 8 bit

    /* filtering code copied from scapy */
    tmp = (tmp >> 16) + (tmp & 0xffff);
    tmp = tmp + (tmp >> 16);
    tmp = ~tmp;
    meta.v4sum = (bit<16>) ((((tmp>>8) & 0xff)|tmp<<8) & 0xffff);

}

action v6sum() {
    bit<32> tmp = 0;
    bit<32> carryover = 0;

    tmp = tmp + (bit<32>) hdr.ipv6.src_addr[15:0];              // 16 bit
    tmp = tmp + (bit<32>) hdr.ipv6.src_addr[31:16];             // 16 bit
    tmp = tmp + (bit<32>) hdr.ipv6.src_addr[47:32];             // 16 bit
    tmp = tmp + (bit<32>) hdr.ipv6.src_addr[63:48];             // 16 bit
    tmp = tmp + (bit<32>) hdr.ipv6.src_addr[79:64];             // 16 bit
    tmp = tmp + (bit<32>) hdr.ipv6.src_addr[95:80];             // 16 bit
    tmp = tmp + (bit<32>) hdr.ipv6.src_addr[111:96];            // 16 bit
    tmp = tmp + (bit<32>) hdr.ipv6.src_addr[127:112];           // 16 bit

    tmp = tmp + (bit<32>) hdr.ipv6.dst_addr[15:0];              // 16 bit
    tmp = tmp + (bit<32>) hdr.ipv6.dst_addr[31:16];             // 16 bit
    tmp = tmp + (bit<32>) hdr.ipv6.dst_addr[47:32];             // 16 bit
    tmp = tmp + (bit<32>) hdr.ipv6.dst_addr[63:48];             // 16 bit
    tmp = tmp + (bit<32>) hdr.ipv6.dst_addr[79:64];             // 16 bit
    tmp = tmp + (bit<32>) hdr.ipv6.dst_addr[95:80];             // 16 bit
    tmp = tmp + (bit<32>) hdr.ipv6.dst_addr[111:96];            // 16 bit
    tmp = tmp + (bit<32>) hdr.ipv6.dst_addr[127:112];           // 16 bit

    tmp = tmp + (bit<32>) hdr.ipv6.payload_length;              // 16 bit
    tmp = tmp + (bit<32>) hdr.ipv6.next_header;                 // 8 bit

    /* filtering code copied from scapy */
    tmp = (tmp >> 16) + (tmp & 0xffff);
    tmp = tmp + (tmp >> 16);
    tmp = ~tmp;
    meta.v6sum = (bit<16>) ((((tmp>>8) & 0xff)|tmp<<8) & 0xffff) ;
}


action delta_prepare()
{
    v4sum();
    v6sum();
    if(meta.v6sum > meta.v4sum) {
        meta.headerdiff = meta.v6sum - meta.v4sum;
    } else {
        meta.headerdiff = meta.v4sum - meta.v6sum;
    }
}

action delta_udp_from_v4_to_v6()
{
    delta_prepare();
    hdr.udp.checksum = hdr.udp.checksum + meta.headerdiff;
}

action delta_tcp_from_v4_to_v6()
{
    delta_prepare();
    hdr.tcp.checksum = hdr.tcp.checksum + meta.headerdiff;
}

action delta_ipv4_from_v6_to_v4()
{
    delta_prepare();
    hdr.tcp.checksum = hdr.tcp.checksum + ~meta.headerdiff;
}

#endif